import os
import json

from flask import Blueprint
from flask import render_template
from flask_login import login_required
from flask import flash
from flask import current_app

from shopyoapi.enhance import base_context
from shopyoapi.html import notify_success

control_panel_blueprint = Blueprint(
    "control_panel",
    __name__,
    template_folder="templates",
    url_prefix="/control_panel",
)
all_info = {}

@control_panel_blueprint.route("/")
@login_required
def index():
    context = base_context()

    for module in os.listdir(os.path.join(current_app.config['BASE_DIR'], "modules")):
        if module.startswith("__"):
            continue
        if module not in ["control_panel"]:
            with open(os.path.join(current_app.config['BASE_DIR'], 'modules', module, 'info.json')) as f:
                module_info = json.load(f)
                all_info[module] = module_info

    context["all_info"] = all_info
    flash(notify_success('Notif test'))
    return render_template("control_panel/index.html", **context)



import os
import json

from flask import Blueprint, render_template, redirect, url_for, flash
from flask_login import login_required
from flask_login import login_user
from flask_login import logout_user
from modules.admin.models import User
from shopyoapi.enhance import base_context
from modules.login.forms import LoginForm


dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

login_blueprint = Blueprint(
    "login",
    __name__,
    url_prefix=module_info["url_prefix"],
    template_folder="templates",
)


@login_blueprint.route("/", methods=["GET", "POST"])
def login():
    context = base_context()
    login_form = LoginForm()
    context['form'] = login_form
    if login_form.validate_on_submit():
        username = login_form.username.data
        password = login_form.password.data
        user = User.query.filter_by(username=username).first()
        if user is None or not user.check_hash(password):
            flash("please check your user id and password")
            return redirect(url_for("login.login"))
        login_user(user)
        return redirect(url_for("control_panel.index"))
    return render_template("/login.html", **context)


@login_blueprint.route("/logout")
@login_required
def logout():
    logout_user()
    return redirect("/login")



from shopyoapi.init import db
from sqlalchemy import exists


class Category(db.Model):
    __tablename__ = "category"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), unique=True)
    products = db.relationship(
        "Product", backref="categories", lazy=True, cascade="all, delete"
    )

    def insert(self):
        """Save category to the database"""
        db.session.add(self)
        db.session.commit()

    def update(self):
        """Update category"""
        db.session.commit()

    def delete(self):
        """delete category"""
        db.session.delete(self)
        db.session.commit()

    @classmethod
    def category_exists(cls, name):
        return db.session.query(exists().where(cls.name == name)).scalar()



"""empty message

Revision ID: 51e638dafa33
Revises: 
Create Date: 2020-09-25 23:37:50.830365

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '51e638dafa33'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('appointments',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=100), nullable=True),
    sa.Column('date', sa.String(length=20), nullable=True),
    sa.Column('time', sa.String(length=20), nullable=True),
    sa.Column('active', sa.String(length=20), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('category',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=100), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('name')
    )
    op.create_table('people',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=100), nullable=True),
    sa.Column('phone', sa.Integer(), nullable=True),
    sa.Column('mobile', sa.Integer(), nullable=True),
    sa.Column('email', sa.String(length=100), nullable=True),
    sa.Column('facebook', sa.String(length=128), nullable=True),
    sa.Column('twitter', sa.String(length=128), nullable=True),
    sa.Column('linkedin', sa.String(length=128), nullable=True),
    sa.Column('age', sa.Integer(), nullable=True),
    sa.Column('birthday', sa.String(length=100), nullable=True),
    sa.Column('notes', sa.String(length=100), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('roles',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(length=100), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('settings',
    sa.Column('setting', sa.String(length=100), nullable=False),
    sa.Column('value', sa.String(length=100), nullable=True),
    sa.PrimaryKeyConstraint('setting')
    )
    op.create_table('users',
    sa.Column('id', sa.String(length=10), nullable=False),
    sa.Column('username', sa.String(length=100), nullable=True),
    sa.Column('password', sa.String(length=128), nullable=True),
    sa.Column('admin_user', sa.Boolean(), nullable=True),
    sa.PrimaryKeyConstraint('id'),
    sa.UniqueConstraint('username')
    )
    op.create_table('product',
    sa.Column('barcode', sa.String(length=100), nullable=False),
    sa.Column('price', sa.Float(), nullable=True),
    sa.Column('name', sa.String(length=100), nullable=True),
    sa.Column('description', sa.String(length=300), nullable=True),
    sa.Column('date', sa.String(length=100), nullable=True),
    sa.Column('in_stock', sa.Integer(), nullable=True),
    sa.Column('discontinued', sa.Boolean(), nullable=True),
    sa.Column('selling_price', sa.Float(), nullable=True),
    sa.Column('category_name', sa.String(length=100), nullable=False),
    sa.ForeignKeyConstraint(['category_name'], ['category.name'], ),
    sa.PrimaryKeyConstraint('barcode')
    )
    op.create_table('role_helpers',
    sa.Column('user_id', sa.Integer(), nullable=True),
    sa.Column('role_id', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['role_id'], ['roles.id'], ),
    sa.ForeignKeyConstraint(['user_id'], ['users.id'], )
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('role_helpers')
    op.drop_table('product')
    op.drop_table('users')
    op.drop_table('settings')
    op.drop_table('roles')
    op.drop_table('people')
    op.drop_table('category')
    op.drop_table('appointments')
    # ### end Alembic commands ###



import shutil
import os


def trycopytree(source, dest):
    try:
        shutil.copytree(source, dest)
        print("done copying {} to {}".format(source, dest))
    except Exception as e:
        print(e)


def trycopy(source, dest):
    try:
        shutil.copy(source, dest)
        print("done copying {} to {}".format(source, dest))
    except Exception as e:
        print(e)


def trymkdir(path):
    try:
        os.mkdir(path)
        print('created dir at', path)
    except Exception as e:
        print(e)


def trymkfile(path, content):
    try:
        with open(path, 'w+') as f:
            f.write(content)
        print('file created at {}'.format(path))
        print('with content:')
        print(content)
    except Exception as e:
        print(e)



from shopyoapi.init import db


class Product(db.Model):
    __tablename__ = "product"
    barcode = db.Column(db.String(100), primary_key=True)
    price = db.Column(db.Float)
    name = db.Column(db.String(100))
    description = db.Column(db.String(300))
    date = db.Column(db.String(100))
    in_stock = db.Column(db.Integer)
    discontinued = db.Column(db.Boolean)
    selling_price = db.Column(db.Float)
    category_name = db.Column(
        db.String(100), db.ForeignKey("category.name"), nullable=False
    )



import os
import shutil
import sys
import subprocess
import json

from .utils import trycopytree
from .utils import trycopy
from .utils import trymkdir
from .utils import trymkfile
from .uploads import add_admin
from .uploads import add_setting


def new_project(path, newfoldername):
    newfoldername = newfoldername.strip('/').strip('\\')
    print("creating new project {}".format(newfoldername))

    base_path = path + "/" + newfoldername
    trymkdir(base_path)
    print("created dir {} in {}".format(newfoldername, path))

    trycopytree("./static", base_path + "/static")
    trycopytree("./tests", base_path + "/tests")
    trycopytree("./modules/base", base_path + "/modules/base")
    trycopytree("./modules/admin", base_path + "/modules/admin")
    trycopytree("./modules/login", base_path + "/modules/login")
    trycopytree("./modules/settings", base_path + "/modules/settings")
    trycopytree("./shopyoapi", base_path + "/shopyoapi")

    trycopy("app.py", base_path + "/app.py")
    trycopy("config.json", base_path + "/config.json")
    trycopy("config.py", base_path + "/config.py")
    trycopy("manage.py", base_path + "/manage.py")


def clean():
    if os.path.exists("shopyo.db"):
        os.remove("shopyo.db")
        print("shopyo.db successfully deleted")
    else:
        print("shopyo.db doesn't exist")
    if os.path.exists("__pycache__"):
        shutil.rmtree("__pycache__")
        print("__pycache__ successfully deleted")
    else:
        print("__pycache__ doesn't exist")
    if os.path.exists("migrations"):
        shutil.rmtree("migrations")
        print("migrations successfully deleted")
    else:
        print("migrations folder doesn't exist")


def initialise():
    with open("config.json", "r") as config:
        config = json.load(config)
    print("Creating Db")
    print("#######################")
    subprocess.run(
        [sys.executable, "manage.py", "db", "init"], stdout=subprocess.PIPE
    )
    print("Migrating")
    print("#######################")
    subprocess.run(
        [sys.executable, "manage.py", "db", "migrate"], stdout=subprocess.PIPE
    )
    subprocess.run(
        [sys.executable, "manage.py", "db", "upgrade"], stdout=subprocess.PIPE
    )

    print("Initialising User")
    print("#######################")
    add_admin(
        config["admin_user"]["name"],
        config["admin_user"]["password"]
    )

    print("Initialising Settings")
    print("#######################")
    for name, value in config["settings"].items():
        add_setting(name, value)
    print("Done!")


def create_module(modulename):
    print('creating module: {}'.format(modulename))
    base_path = 'modules/' + modulename
    trymkdir(base_path)
    trymkdir(base_path+'/templates')
    trymkdir(base_path+'/templates/'+modulename)
    view_content = '''
from flask import Blueprint
{0}_blueprint = Blueprint(
    "{0}",
    __name__,
    url_prefix='/{0}',
    template_folder="templates",
)


@{0}_blueprint.route("/")
def index():
    return ''
'''.format(modulename)
    trymkfile(base_path+'/'+'view.py', view_content)
    trymkfile(base_path+'/'+'forms.py', '')
    trymkfile(base_path+'/'+'models.py', '')
    info_json_content = '''{{
        "name": "{}",
        "type": "show",
        "fa-icon": "fa fa-store",
        "url_prefix": "/{}"
}}'''.format(modulename.capitalize(), modulename)
    trymkfile(base_path+'/'+'info.json', info_json_content)



from flask import Blueprint
from flask import render_template
from flask import request
from flask import redirect
from flask import jsonify
from modules.product.models import Product

from shopyoapi.init import db, ma

from flask_login import login_required

from shopyoapi.enhance import base_context
from sqlalchemy import exists

product_blueprint = Blueprint(
    "prods", __name__, template_folder="templates", url_prefix="/prods"
)

import uuid

class Productchema(ma.Schema):
    class Meta:
        # Fields to expose
        fields = (
            "barcode",
            "name",
            "description",
            "category",
            "price",
            "selling_price",
            "in_stock",
            "discontinued",
        )


product_schema = Productchema()
product_schema = Productchema(many=True)


@product_blueprint.route("/list_prods/<category_name>")
@login_required
def list_prods(category_name):
    context = base_context()
    products = Product.query.filter(
        Product.category_name == category_name
    ).all()
    context["products"] = products
    context["category"] = category_name

    return render_template("prods/list.html", **context)


@product_blueprint.route("/add/<category_name>", methods=["GET", "POST"])
@login_required
def prods_add(category_name):
    context = base_context()

    has_product = False
    if request.method == "POST":
        barcode = request.form["barcode"]
        name = request.form["name"]
        description = request.form["description"]
        date = request.form["date"]
        price = request.form["price"]
        selling_price = request.form["selling_price"]
        in_stock = request.form["in_stock"]
        if request.form["discontinued"] == "True":
            discontinued = True
        else:
            discontinued = False

        # category = Category.query.filter(
        #     Category.name == category_name).first()
        # print(category, category_name, category.name)
        has_product = db.session.query(
            exists().where(Product.barcode == barcode)
        ).scalar()

        if has_product is False:
            p = Product(
                barcode=barcode,
                name=name,
                in_stock=in_stock,
                category_name=category_name,
                discontinued=discontinued
            )
            if description:
                p.description = description.strip()
            if date:
                p.date = date.strip()
            if price:
                p.price = price.strip()
            if selling_price:
                p.selling_price = selling_price.strip()
            
            db.session.add(p)
            db.session.commit()
        context["category"] = category_name
        context["has_product"] = str(has_product)
        return render_template("prods/add.html", **context)

    context["category"] = category_name
    context["has_product"] = str(has_product)
    context['barcodestr'] = uuid.uuid1()
    return render_template("prods/add.html", **context)


@product_blueprint.route(
    "/delete/<category_name>/<barcode>", methods=["GET", "POST"]
)
@login_required
def prods_delete(category_name, barcode):
    Product.query.filter(
        Product.barcode == barcode and Product.category == category_name
    ).delete()
    db.session.commit()
    return redirect("/prods/list_prods/{}".format(category_name))


@product_blueprint.route(
    "/edit/<category_name>/<barcode>", methods=["GET", "POST"]
)
@login_required
def prods_edit(category_name, barcode):
    context = base_context()

    product = Product.query.filter(
        Product.barcode == barcode and Product.category == category_name
    ).first()

    context["product"] = product
    context["category"] = category_name
    return render_template("prods/edit.html", **context)


@product_blueprint.route("/update", methods=["GET", "POST"])
@login_required
def prods_update():
    # this block is only entered when the form is submitted
    if request.method == "POST":
        barcode = request.form["barcode"]
        old_barcode = request.form["old_barcode"]
        category = request.form["category"]

        name = request.form["name"]
        description = request.form["description"]
        category = request.form["category"]
        date = request.form["date"]
        price = request.form["price"]
        selling_price = request.form["selling_price"]
        in_stock = request.form["in_stock"]
        if request.form["discontinued"] == "True":
            discontinued = True
        else:
            discontinued = False

        p = Product.query.filter(
            Product.barcode == old_barcode
            and Product.category == category
        ).first()
        p.barcode = barcode
        p.name = name
        p.description = description
        p.category = category
        p.date = date
        p.price = price
        p.selling_price = selling_price
        p.in_stock = in_stock
        p.discontinued = discontinued
        p.category = category
        db.session.commit()
        return redirect("/prods/list_prods/{}".format(category))


@product_blueprint.route("/lookup/<category_name>")
@login_required
def lookup_prods(category_name):
    context = base_context()

    context["category"] = category_name
    context["fields"] = [
        key.replace("_", " ")
        for key in Product.__table__.columns.keys()
        if key not in ["category_name"]
    ]

    return render_template("prods/lookup.html", **context)


# api
@product_blueprint.route(
    "/search/<category_name>/barcode/<user_input>", methods=["GET"]
)
@login_required
def search(category_name, user_input):
    if request.method == "GET":
        print(request.args["field"], request.args["global_search"])
        field = request.args["field"]
        global_search = request.args["global_search"]
        if global_search == "True":
            all_p = Product.query.filter(
                (getattr(Product, field).like("%" + user_input + "%"))
                & (Product.category_name == category_name)
            ).all()
            result = product_schema.dump(all_p)
        else:
            all_p = Product.query.filter(
                getattr(Product, field).like("%" + user_input + "%")
            ).all()
            result = product_schema.dump(all_p)
    return jsonify(result)


# api
@product_blueprint.route("/check/<barcode>", methods=["GET"])
@login_required
def check(barcode):
    has_product = db.session.query(
        exists().where(Product.barcode == barcode)
    ).scalar()
    return jsonify({"exists": has_product})



import importlib
import os

from flask import Flask, redirect
from flask_wtf.csrf import CSRFProtect
from shopyoapi.init import db, login_manager, ma
from config import app_config

base_path = os.path.dirname(os.path.abspath(__file__))
def create_app(config_name):
    app = Flask(__name__)
    configuration = app_config[config_name]
    app.config.from_object(configuration)

    db.init_app(app)
    ma.init_app(app)
    login_manager.init_app(app)
    csrf = CSRFProtect(app)  # noqa
    
    for module in os.listdir(os.path.join(base_path, "modules")):
        if module.startswith("__"):
            continue
        mod = importlib.import_module("modules.{}.view".format(module))
        app.register_blueprint(getattr(mod, "{}_blueprint".format(module)))

    @app.route("/")
    def index():
        return redirect(configuration.HOMEPAGE_URL)

    return app

app = create_app("development")


if __name__ == "__main__":
    
    app.run(debug=True, host="0.0.0.0")



import os
import json

from flask import Blueprint
from flask import render_template
from flask import request
from flask import redirect
from flask import jsonify
from flask_sqlalchemy import sqlalchemy
from modules.category.models import Category


from flask_login import login_required

from shopyoapi.enhance import base_context
from shopyoapi.enhance import get_setting


dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

category_blueprint = Blueprint(
    "category",
    __name__,
    template_folder="templates",
    url_prefix=module_info["url_prefix"],
)


@category_blueprint.route("/")
@login_required
def category():
    context = base_context()
    context["categorys"] = Category.query.all()
    context["active_page"] = get_setting("SECTION_NAME")
    return render_template("category/index.html", **context)


@category_blueprint.route("/add", methods=["GET", "POST"])
@login_required
def category_add():
    context = base_context()

    has_category = False
    if request.method == "POST":
        name = request.form["name"]
        has_category = Category.category_exists(name)
        if has_category is False:
            m = Category(name=name)
            m.insert()
        return render_template("category/add.html", **context)

    context["has_category"] = str(has_category)
    return render_template("category/add.html", **context)


@category_blueprint.route("/delete/<name>", methods=["GET", "POST"])
@login_required
def category_delete(name):
    category = Category.query.filter(Category.name == name).first()
    category.delete()
    return redirect("/category")


@category_blueprint.route("/update", methods=["GET", "POST"])
@login_required
def category_update():
    context = base_context()

    if (
        request.method == "POST"
    ):  # this block is only entered when the form is submitted
        name = request.form["category_name"]
        old_name = request.form["old_category_name"]
        try:
            m = Category.query.filter_by(name=old_name).first()
            m.name = name
            m.update()
        except sqlalchemy.exc.IntegrityError:
            context[
                "message"
            ] = "you cannot modify to an already existing category"
            context["redirect_url"] = "/category/"
            render_template("category/message.html", **context)
        return redirect("/category/")


@category_blueprint.route("/edit/<category_name>", methods=["GET", "POST"])
@login_required
def category_edit(category_name):
    context = base_context()
    context["category_name"] = category_name
    return render_template("category/edit.html", **context)


# api
@category_blueprint.route("/check/<category_name>", methods=["GET"])
@login_required
def check(category_name):
    has_category = Category.category_exists(category_name)
    return jsonify({"exists": has_category})



from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField, SubmitField
from wtforms.validators import DataRequired


class LoginForm(FlaskForm):
    username = StringField(
        'name', [DataRequired()], render_kw={
            "class": "form-control", "autocomplete": "off"
            })
    password = PasswordField('Password', [DataRequired()],
                             render_kw={"class": "form-control",
                                        "autocomplete": "off"})
    submit = SubmitField('Login', [DataRequired()],
                         render_kw={"class": "btn btn-info"})



import sys

from flask_migrate import Migrate
from flask_migrate import MigrateCommand
from flask_script import Manager

from shopyoapi.init import db
from app import app

from shopyoapi.cmd import new_project
from shopyoapi.cmd import clean
from shopyoapi.cmd import initialise
from shopyoapi.cmd import create_module
from shopyoapi.database import autoload_models

migrate = Migrate(app, db, compare_type=True)
manager = Manager(app)

manager.add_command("db", MigrateCommand)


def runserver():
    app.run()


def rundebug():
    app.run(debug=True, host="0.0.0.0")


def custom_commands(args):
    # non migration commands
    if args[1] != "db":
        if args[1] == "initialise":
            autoload_models()
            initialise()
        elif args[1] == "clean":
            clean()
        elif args[1] == "runserver":
            runserver()
        elif args[1] == "rundebug":
            rundebug()
        elif args[1] == "test":
            print("test ok")
        elif args[1] == "new" and args[2] and args[3]:
            # new <path> <folder name>
            new_project(args[2], args[3])
        elif args[1] == 'startapp' and args[2]:
            create_module(args[2])
        sys.exit()
    elif args[1] == "db":
        autoload_models()


if __name__ == "__main__":
    custom_commands(sys.argv)
    manager.run()



from modules.settings.models import Settings


def get_setting(name):
    s = Settings.query.get(name)
    return s.value


def base_context():
    base_context = {
        "APP_NAME": get_setting("APP_NAME"),
        "SECTION_NAME": get_setting("SECTION_NAME"),
        "SECTION_ITEMS": get_setting("SECTION_ITEMS"),
    }
    return base_context.copy()



'''
Used on flash
flash(notify_success('mail sent!'))
'''
def notify(message, alert_type="primary"):
    '''
    secondary, success, danger, warning, info, light
    dark
    '''
    alert = """
    <div class="alert alert-{alert_type} alert-dismissible fade show" role="alert"
        style="opacity: 0.98;">
      {message}

      <button type="button" class="close" data-dismiss="alert" aria-label="Close">
        <span aria-hidden="true">&times;</span>
      </button>
    </div>
    """.format(message=message, alert_type=alert_type)
    return alert

def notify_success(message):
    return notify(message, alert_type='success')

def notify_danger(message):
    return notify(message, alert_type='danger')

def notify_warning(message):
    return notify(message, alert_type='warning')

def notify_info(message):
    return notify(message, alert_type='info')


from flask import flash, redirect, url_for
from flask_login import current_user
from functools import wraps

from shopyoapi.init import login_manager
from modules.admin.models import User

login_manager.login_view = "login.login"
login_manager.login_message = "Please login for access"


@login_manager.user_loader
def load_user(id):
    return User.query.get(id)


def admin_required(f):
    @wraps(f)
    def wrap(*args, **kwargs):
        if current_user.admin_user is True:
            return f(*args, **kwargs)
        else:
            flash("You need to be an admin to view this page.")
            return redirect(url_for("category.category"))

    return wrap



from werkzeug.security import generate_password_hash

from .init import db
from app import app

from ..modules.admin.models import User
from ..modules.admin.models import Role
from ..modules.settings.models import Settings


def add_admin(username, password):
    with app.app_context():
        user = User()
        user.username = username
        user.password = generate_password_hash(password, method="sha256")
        user.admin_user = True
        user.insert()


def add_setting(name, value):
    with app.app_context():
        if Settings.query.filter_by(setting=name).first():
            s = Settings.query.get(name)
            s.value = value
            db.session.commit()
        else:
            s = Settings(setting=name, value=value)
            db.session.add(s)
            db.session.commit()



import os
import importlib

def autoload_models():
    print('Auto importing models')
    for module in os.listdir("modules"):
        if module.startswith("__"):
            continue
        try:
            to_load = "modules.{}.models".format(module)
            importlib.import_module(to_load)
            print('[x]', 'imported', to_load)
        except Exception as e:
            print('[ ]', e)



import pytest
from modules.admin.models import User
from app import create_app
from shopyoapi.init import db
import json

from shopyoapi.uploads import add_setting
from modules.settings.models import Settings
# from shopyoapi.cmd import initialise

# run in shopyo/shopyo
# python -m pytest tests/

import os
if os.path.exists("testing.db"):
    os.remove("testing.db")

@pytest.fixture(scope='module')
def new_user():
    user = User(username='abcd')
    user.set_hash('pass')
    return user


@pytest.fixture(scope='module')
def test_client():
    flask_app = create_app('testing')
 
    # Flask provides a way to test your application by exposing the Werkzeug test Client
    # and handling the context locals for you.
    testing_client = flask_app.test_client()
 
    # Establish an application context before running the tests.
    ctx = flask_app.app_context()
    ctx.push()
 
    yield testing_client  # this is where the testing happens!
 
    ctx.pop()

@pytest.fixture(scope='module')
def init_database():
    
    # Create the database and the database table
    db.create_all()
 
    # Insert user data
    user1 = User(username='car')
    user1.set_hash('pass')
    user2 = User(username='field')
    user2.set_hash('pass')
    db.session.add(user1)
    db.session.add(user2)

    with open("config.json", "r") as config:
        config = json.load(config)
    for name, value in config["settings"].items():
        s = Settings(setting=name, value=value)
        db.session.add(s)
    db.session.commit()
 
    # Commit the changes for the users
    db.session.commit()
    
    # initialise()
    yield db  # this is where the testing happens!
 
    # db.drop_all()


"""
.. module:: AdminViews
   :synopsis: All endpoints of the admin views are defined here.

"""
import os
import json
from config import Config
from flask import Blueprint, render_template, request, redirect
from flask import url_for

from flask_login import login_required
from shopyoapi.init import db

from shopyoapi.enhance import base_context
from sqlalchemy import exists

from modules.admin.admin import admin_required
from modules.admin.models import User
from modules.admin.models import Role

dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

admin_blueprint = Blueprint(
    "admin",
    __name__,
    template_folder="templates",
    url_prefix=module_info["url_prefix"],
)


@admin_blueprint.route("/")
@login_required
@admin_required
def user_list():
    """
           **Get The List of User**

            Lists all users in the database.

    """
    context = base_context()
    context["users"] = User.query.all()
    return render_template("admin/index.html", **context)


@admin_blueprint.route("/add", methods=["GET", "POST"])
@login_required
@admin_required
def user_add():
    """
               **Adds a User**

            adds a user to database.

    """
    context = base_context()
    if request.method == "POST":
        username = request.form["name"]
        password = request.form["password"]
        admin_user = request.form.get("admin_user")
        if admin_user == "True":
            admin_user = True
        else:
            admin_user = False

        has_user = db.session.query(
            exists().where(User.username == username)).scalar()

        if has_user is False:
            new_user = User()
            new_user.username = username
            new_user.admin_user = admin_user
            new_user.set_hash(password)
            
            for key in request.form:
                if key.startswith('role_'):
                    roleid = key.split('_')[1]
                    role = Role.query.get(roleid)
                    new_user.roles.append(role)
            new_user.insert()
            return redirect(url_for('admin.user_add'))

    context['roles'] = Role.query.all()
    return render_template("admin/add.html", **context)


@admin_blueprint.route("/delete/<id>", methods=["GET"])
@login_required
@admin_required
def admin_delete(id):
    """
                   **Delete a User**

        :param id: id of the user
        :type id: int

    """
    user = User.query.get(id)
    user.delete()
    return redirect("/admin")


@admin_blueprint.route("/edit/<id>", methods=["GET"])
@login_required
@admin_required
def admin_edit(id):
    """
                   **Update information for a User**

        :param id: id of the user
        :type id: int

    """
    context = base_context()
    user = User.query.get(id)
    context['user'] = user
    context['user_roles'] = [r.name for r in user.roles]
    context['roles'] = Role.query.all()
    return render_template("admin/edit.html", **context)


@admin_blueprint.route("/update", methods=["POST"])
@login_required
@admin_required
def admin_update():
    """
                   **Update a User record**

    """
    id = request.form["id"]
    password = request.form["password"]
    username = request.form["username"]
    admin_user = request.form.get("admin_user")
    if admin_user == "True":
        admin_user = True
    else:
        admin_user = False
    user = User.query.get(id)
    user.set_hash(password)
    user.admin_user = admin_user
    user.username = username
    if password.strip():
        user.set_hash(password)
    user.roles[:] = []
    for key in request.form:
        if key.startswith('role_'):
            roleid = key.split('_')[1]
            role = Role.query.get(roleid) 
            user.roles.append(role)

    user.update()
    return redirect("/admin")

@admin_blueprint.route("/roles")
@login_required
@admin_required
def roles():
    context = base_context()
    context['roles'] = Role.query.all()
    return render_template('admin/roles.html', **context)

@admin_blueprint.route("/roles/add", methods=['POST'])
@login_required
@admin_required
def roles_add():
    if request.method == 'POST':
        if not Role.query.filter(
                  Role.name == request.form['name']
              ).first():
            role = Role(name=request.form['name'])
            role.insert()
    return redirect(url_for('admin.roles'))


@admin_blueprint.route("/roles/<role_id>/delete", methods=['GET'])
@login_required
@admin_required
def roles_delete(role_id):
    role = Role.query.get(role_id)
    role.delete()
    return redirect(url_for('admin.roles'))


@admin_blueprint.route("/roles/update", methods=['POST'])
@login_required
@admin_required
def roles_update():
    if request.method == 'POST':
        role = Role.query.get(request.form['role_id'])
        role.name = request.form['role_name']
        role.update()
    return redirect(url_for('admin.roles'))


"""
.. module:: AdminModels
   :synopsis: Contains model of a user Record

"""

from werkzeug.security import generate_password_hash, check_password_hash
from shopyoapi.init import db
from flask_login import UserMixin
from uuid import uuid4


role_helpers = db.Table('role_helpers',
    db.Column('user_id', db.Integer, db.ForeignKey('users.id')),
    db.Column('role_id', db.Integer, db.ForeignKey('roles.id'))
)


class User(UserMixin, db.Model):
    """ User model """

    __tablename__ = "users"
    id = db.Column(db.String(10), primary_key=True)
    username = db.Column(db.String(100), unique=True)
    password = db.Column(db.String(128))
    
    admin_user = db.Column(db.Boolean, default=False)
    roles = db.relationship("Role",
        secondary=role_helpers, 
        cascade = "all, delete")

    def __init__(self, **kwargs):
        super(User, self).__init__(**kwargs)
        self.id = str(uuid4())

    def set_hash(self, password):
        self.password = generate_password_hash(password, method="sha256")

    def check_hash(self, password):
        return check_password_hash(self.password, password)

    def insert(self):
        db.session.add(self)
        db.session.commit()

    def update(self):
        db.session.commit()

    def delete(self):
        db.session.delete(self)
        db.session.commit()

    def __repr__(self):
        return f'User({self.username!r})'


class Role(db.Model):

    __tablename__ = "roles"
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100))

    def insert(self):
        db.session.add(self)
        db.session.commit()

    def update(self):
        db.session.commit()

    def delete(self):
        db.session.delete(self)
        db.session.commit()





import os


base_path = os.path.dirname(os.path.abspath(__file__))

class Config:
    """Parent configuration class."""

    DEBUG = False
    SQLALCHEMY_DATABASE_URI = "sqlite:///shopyo.db"
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    SECRET_KEY = "qow32ijjdkc756osk5dmck"  # Need a generator
    HOMEPAGE_URL = "/control_panel"
    BASE_DIR = base_path


class DevelopmentConfig(Config):
    """Configurations for development"""

    ENV = "development"
    DEBUG = True


class TestingConfig(Config):
    """Configurations for testsing"""

    SQLALCHEMY_DATABASE_URI = "sqlite:///testing.db"
    DEBUG = True
    LIVESERVER_PORT = 8943
    LIVESERVER_TIMEOUT = 10
    
    BCRYPT_LOG_ROUNDS = 4
    TESTING = True
    WTF_CSRF_ENABLED = False

app_config = {
    "development": DevelopmentConfig,
    "production": Config,
    "testing": TestingConfig,
}



import os
import json

from flask import Blueprint
from flask import render_template
from flask import request
from shopyoapi.init import db
from modules.settings.models import Settings

from flask_login import login_required

from shopyoapi.enhance import base_context


dirpath = os.path.dirname(os.path.abspath(__file__))
module_info = {}

with open(dirpath + "/info.json") as f:
    module_info = json.load(f)

settings_blueprint = Blueprint(
    "settings",
    __name__,
    template_folder="templates",
    url_prefix=module_info["url_prefix"],
)


@settings_blueprint.route("/")
@login_required
def settings_main():
    context = base_context()

    settings = Settings.query.all()

    context["settings"] = settings
    return render_template("settings/index.html", **context)


@settings_blueprint.route("/edit/<settings_name>", methods=["GET", "POST"])
@login_required
def settings_edit(settings_name):
    context = base_context()

    s = Settings.query.get(settings_name)

    context["settings_name"] = settings_name
    context["current_value"] = s.value

    return render_template("settings/edit.html", **context)


@settings_blueprint.route("/update", methods=["GET", "POST"])
@login_required
def settings_update():
    context = base_context()

    settings_name = request.form["settings_name"]
    settings_value = request.form["settings_value"]
    s = Settings.query.get(settings_name)
    s.value = settings_value
    db.session.commit()
    settings = Settings.query.all()

    context["settings"] = settings
    return render_template("settings/index.html", **context)



def test_new_user(new_user):
    assert new_user.username == 'abcd'
    assert new_user.password != 'pass'
    assert not new_user.admin_user


def test_home_page(test_client):
    response = test_client.get('/')
    assert response.status_code == 302 # redirect



def test_valid_login_logout(test_client, init_database):
    response = test_client.post('/login/',
                                data=dict(username='car', password='pass'),
                                follow_redirects=True)
    #print(response.data)
    assert response.status_code == 200
    # print(response.data)
    assert b"panel" in response.data # still only showing normal login screen
    # Control panel
    # response = test_client.get('/logout', follow_redirects=True)
    # assert response.status_code == 200

def test_control_panel(test_client, init_database):
    response = test_client.get('/control_panel/', follow_redirects=True)
    print(response.data)
    assert response.status_code == 200



import os
import json

from flask import Blueprint
from flask import render_template
from flask_login import login_required

from shopyoapi.enhance import base_context

internals_blueprint = Blueprint(
    "internals",
    __name__,
    url_prefix='/internals',
    template_folder="templates",
)
all_info = {}

@internals_blueprint.route("/")
@login_required
def index():
    context = base_context()

    for module in os.listdir("modules"):
        if module.startswith("__"):
            continue
        if module not in ["control_panel"]:
            with open("modules/{}/info.json".format(module)) as f:
                module_info = json.load(f)
                all_info[module] = module_info

    context["all_info"] = all_info
    return render_template("internals/index.html", **context)



import sys

from .shopyoapi.cmd import new_project

def main():
    args = sys.argv
    if args[1] == "new" and len(args) == 4:
        new_project(args[2], args[3])


if __name__ == "__main__":
    main()


from userapi.html import notify_warning
from flask import flash

def flash_errors(form):
    """Flashes form errors"""
    for field, errors in form.errors.items():
        for error in errors:
            error_msg = u"Error in the %s field - %s" % (
                getattr(form, field).label.text,
                error
            )
            flash(notify_warning(error_msg))